﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Data.SqlClient;
using System.Data;
using System.Data.Linq.SqlClient;
using System.Data.Linq;
using GT.Comun;
using System.Data.Common;

namespace GT.AccesoDatos
{
    public class TerminoAD : BaseAD
    {

        #region Consulta

        /// <summary>
        /// Obtiene los terminos de un area
        /// </summary>
        /// <param name="idArea">id del area</param>
        /// <returns>BindingList de Termino</returns>
        public BindingList<Termino> ObtenerTerminosPorAreaId(int idArea)
        {
            EntidadesDataContext dc = GetDC();
            BindingList<Termino> resultado = new BindingList<Termino>();
            using (SqlConnection cnn = (SqlConnection)dc.Connection)
            {
                SqlCommand cmd = new SqlCommand("ObtenerTerminosPorClaveId", cnn);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("idArea", idArea);

                cnn.Open();
                SqlDataReader reader = cmd.ExecuteReader(CommandBehavior.CloseConnection);
                while (reader.Read())
                {
                    Termino itm = new Termino();
                    itm.IdTermino=(int)reader["IdTermino"];
                    itm.Nombre = (string)reader["Nombre"];
                    itm.Descripcion = (string)reader["Descripcion"];
                    itm.Consultado = (int)reader["Consultado"];
                    resultado.Add(itm);
                }
                reader.Close();
                foreach (Termino itm in resultado)
                {
                    itm.TerminoArea.AddRange(dc.TerminoArea.Where(tera => tera.IdTermino == itm.IdTermino));
                }
            }
            return resultado;

        }
        
        /// <summary>
        /// Obtiene los terminos por un like del nombre
        /// </summary>
        /// <param name="nombre">literal de busqueda</param>
        /// <returns>BindingList de Termino</returns>
        public BindingList<Termino> ObtenerTerminosPorLike(String nombre)
        {
            IQueryable<Termino> query;
            EntidadesDataContext dc = GetDC();
            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Termino>(termino => termino.TerminoArea);
            dc.LoadOptions = loadOptions;
            query = from termino in dc.Termino where termino.Nombre.Contains(nombre) select termino;
            return QueryToBindingList(query);

        }


        /// <summary>
        /// Obtiene un Termino por su identificador
        /// </summary>
        /// <param name="idTermino">Identificador de Termino</param>
        /// <returns>Termino</returns>
        public Termino ObtenerTerminoPorId(int idTermino)
        {
            
            EntidadesDataContext dc = GetDC();
            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Termino>(termino => termino.TerminoArea);
            dc.LoadOptions = loadOptions;
            Termino res = dc.Termino.Where(term => term.IdTermino == idTermino).SingleOrDefault();
            res.Consultado = res.Consultado + 1;//incremento el contador de vistas
            dc.SubmitChanges();//actualizo el contador
            return res;
            

        }

        /// <summary>
        /// Obtiene todos los terminos
        /// </summary>
        /// <returns>BindingList de Termino</returns>
        public BindingList<Termino> ObtenerTerminos()
        {
            IQueryable<Termino> query;
            EntidadesDataContext dc = GetDC();
            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Termino>(termino => termino.TerminoArea);
            loadOptions.LoadWith<Area>(area => area.TerminoArea);
            dc.LoadOptions = loadOptions;
            query = from termino in dc.Termino select termino;
            return QueryToBindingList(query);
        }

        #endregion

        #region Mantenimientos


        /// <summary>
        /// Añade un termino
        /// </summary>
        /// <param name="termino">termino a añadir</param>
        /// <returns>Id del termino añadido</returns>
        public int AgregarTermino(Termino termino)
        {
            EntidadesDataContext dc = GetDC();
            dc.Termino.InsertOnSubmit(termino);
            dc.SubmitChanges();
            return termino.IdTermino;
        }

        /// <summary>
        /// Asocia un termino con un area 
        /// </summary>
        /// <param name="termino">id del termino a añadir</param>
        /// <param name="idArea">id del area a la que pertenece el termino</param>
        public void AsignarTerminoToArea(int idtermino, int idArea)
        {
            EntidadesDataContext dc = GetDC();
            TerminoArea registro = new TerminoArea();
            Area areaParent = dc.Area.Single(x => x.IdArea == idArea);
            Termino terminoParent = dc.Termino.Single(x => x.IdTermino == idtermino);
            registro.Area = areaParent;
            registro.Termino = terminoParent;
            dc.TerminoArea.InsertOnSubmit(registro);
            dc.SubmitChanges(); 
        }

        /// <summary>
        /// Quita la relación entre un termino y un area 
        /// </summary>
        /// <param name="termino">id del termino</param>
        /// <param name="idArea">id del area</param>
        public void DesAsignarTerminoToArea(int idtermino, int idArea)
        {
            EntidadesDataContext dc = GetDC();
            dc.TerminoArea.DeleteOnSubmit(dc.TerminoArea.Where(tera=> 
                tera.IdArea==idArea && tera.IdTermino==idtermino).SingleOrDefault());
            dc.SubmitChanges();
        }

        /// <summary>
        /// Quita todas las areas asociadas (de la relación) a un termino 
        /// </summary>
        /// <param name="idtermino">id del termino</param>
        public void DesAsignarTerminoToAreas(int idtermino)
        {
            EntidadesDataContext dc = GetDC();
            var areasBorrar =
            from terminoarea in dc.TerminoArea
            where terminoarea.IdTermino == idtermino
            select terminoarea;
            foreach (var areatermino in areasBorrar)
                dc.TerminoArea.DeleteOnSubmit(areatermino);
            dc.SubmitChanges();
        }



        /// <summary>
        /// Elimina un termino del sistema
        /// </summary>
        /// <param name="idTermino">id del termino</param>
        public void EliminarTermino(int idTermino)
        {
            EntidadesDataContext dc = GetDC();
            dc.Termino.DeleteOnSubmit(dc.Termino.Where(termino => termino.IdTermino 
                == idTermino).SingleOrDefault());
            dc.SubmitChanges();
            
        }

        /// <summary>
        /// Modifica la información de un Termino
        /// </summary>
        /// <param name="terminoMod">Termino modificado</param>
        public void ModificarTermino(Termino terminoMod)
        {
            EntidadesDataContext dc = GetDC();
            Termino term = dc.Termino.Where(termino => termino.IdTermino == terminoMod.IdTermino).SingleOrDefault();
            term.Descripcion = terminoMod.Descripcion;
            term.Nombre = terminoMod.Nombre;
            dc.SubmitChanges();

        }
        
        
        #endregion
    }
}
