﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GT.Comun;
using GT.Comun.Helpers;
using GT.AccesoDatos;
using System.ComponentModel;
using System.Security.Principal;
using System.Threading;
using System.Security.Permissions;

namespace GT.LogicaNegocio
{
    public class AreaCN
    {

        #region consulta


        /// <summary>
        /// Obtiene toda la información de areas en relación a un usuario dado
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="todas">BindingList de todas las Areas</param>
        /// <param name="solicitadas">BindingList de todas las Areas solicitadas</param>
        /// <param name="experto">BindingList de todas las Areas en las que es experto</param>
        /// <param name="idUsuario">Identificador del usuario</param>
        /// <returns>Lista de tipos de area</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public void ObtenerInfoAreasUsuarioId(ref BindingList<Area> todas,
             ref BindingList<Area> solicitadas, ref BindingList<Area> experto, int idUsuario)
        {
            AreaAD areaAD = new AreaAD();
            todas = areaAD.ObtenerAreas();
            experto = areaAD.ObtenerAreasPorExperto(idUsuario);
            solicitadas = areaAD.ObtenerAreasUsuarioPendiente(idUsuario);
        }

        /// <summary>
        /// Obtiene las areas (filtrado opcionalmente por un like del nombre)
        /// Seguridad: Publico
        /// </summary>
        /// <param name="nombre">Nombre del area</param>
        /// <returns>BindingList de Area</returns>
        public BindingList<Area> ObtenerAreas(String nombre)
        {
            AreaAD areaAD = new AreaAD();
            BindingList<Area> resultado;
            if (nombre==""){
                resultado=areaAD.ObtenerAreas();
            }else{
                resultado=areaAD.ObtenerAreaPorLike(nombre);
            }
            return resultado;
        }

        /// <summary>
        /// Obtiene todas las areas
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <returns>BindingList de Area</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public BindingList<Area> ObtenerAreas()
        {
           AreaAD areaAD = new AreaAD();

           return areaAD.ObtenerAreas();
        }

        /// <summary>
        /// Obtiene las areas pendientes con un usuario pendiente de aprobar como experto
        /// Seguridad: Administrador
        /// </summary>
        /// <returns>BindingList de Area</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public BindingList<Area> ObtenerAreasConUsuarioPendiente()
        {
            AreaAD areaAD = new AreaAD();
            return areaAD.ObtenerAreasConUsuarioPendiente();
        }

        /// <summary>
        /// Obtiene las areas que un usuario tiene pendiente de aprobar
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <param name="usuario">Usuario</param>
        /// <returns>BindingList de Area</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public BindingList<Area> ObtenerAreasUsuarioPendiente(String usuario)
        {
            UsuarioAD usuarioAD = new UsuarioAD();
            AreaAD areaAD = new AreaAD();
            return areaAD.ObtenerAreasUsuarioPendiente(usuarioAD.ObtenerUsuarioPorNombre(usuario).IdUsuario);
        }

        /// <summary>
        /// Obtiene las areas pendientes de aprobar para un id de usuario
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="idUsuario">Identificador del Usuario</param>
        /// <returns>BindingList de Area</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public BindingList<Area> ObtenerAreasUsuarioPendiente(int idUsuario)
        {
            AreaAD areaAD = new AreaAD();
            return areaAD.ObtenerAreasUsuarioPendiente(idUsuario);
        }

        /// <summary>
        /// Obtiene las areas en las que un usuario es experto
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <param name="idUsuario">Id del usuario</param>
        /// <returns>BindingList de Area</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public BindingList<Area> ObtenerAreasUsuarioExperto(int idUsuario)
        {
            AreaAD areaAD = new AreaAD();
            return areaAD.ObtenerAreasPorExperto(idUsuario);
        }

        /// <summary>
        /// Obtiene el Area por su identificador
        /// Seguridad: Publico
        /// </summary>
        /// <param name="IdArea">Id del Area</param>
        /// <returns>Area</returns>
        public Area ObtenerAreaPorId(int IdArea)
        {
            AreaAD areaAD = new AreaAD();

            return areaAD.ObtenerAreaPorId(IdArea);
        }

        #endregion


        #region mantenimientos

        /// <summary>
        /// Añade un area al sistema
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="area">Area a añadir</param>
        /// <returns>Identificador del Area</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public int AgregarArea(Area area)
        {
            int res=0;
            AreaAD areaAD = new AreaAD();
            //Agregamos algo de logica de negocio:
            //No se permiten 2 areas con el mismo nombre y tipo
            Area areaExiste = areaAD.ObtenerAreaPorNombreyTipo(area.Nombre, area.Tipo);
            if (areaExiste == null)
                res = areaAD.AgregarArea(area);
            if (areaExiste != null)
                throw new Exception("Ya existe un Area con ese nombre y tipo");
            return res;
        }

        /// <summary>
        /// Modifica un area del sistema
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="area">Area modificada</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public void ModificarArea(Area area)
        {
            AreaAD areaAD = new AreaAD();
            areaAD.ModificarArea(area);
        }

        /// <summary>
        /// Elimina un area del sistema
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="idArea">Identificador del Area</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public void EliminarArea(int idArea)
        {
            AreaAD areaAD = new AreaAD();
            areaAD.EliminarArea(idArea);

        }


        #endregion

       
    }
}
