﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GT.Comun;
using GT.Comun.Helpers;
using GT.AccesoDatos;
using System.ComponentModel;
using System.Security.Principal;
using System.Threading;
using System.Security.Permissions;

namespace GT.LogicaNegocio
{
    /// <summary>
    /// Operaciones de negocio sobre los usuarios
    /// </summary>
    public class UsuarioCN
    {

        #region consultas

        /// <summary>
        /// Obtiene los usuarios filtrado según los criterios:
        /// nombre, email y rol
        /// Seguridad: Administradores
        /// </summary>
        /// <param name="nombre">Nombre del usuario</param>
        /// <param name="email">Email del usuario</param>
        /// <param name="rol">Rol del usuario</param>
        /// <returns>BindingList de Usuario</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public BindingList<Usuario> ObtenerUsuariosFiltradoPor(String nombre, String email, 
            Nullable<Rol> rol)
        {
            UsuarioAD usuariosAD = new UsuarioAD();
            return usuariosAD.ObtenerUsuariosFiltradoPor(nombre, email, rol);
        }


        /// <summary>
        /// Obtiene un usuario por su identificador
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <param name="idUsuario">Id del usuario</param>
        /// <returns>Usuario</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public Usuario ObtenerUsuarioPorId(int idUsuario)
        {
            UsuarioAD usuariosAD = new UsuarioAD();

            return usuariosAD.ObtenerUsuarioPorId(idUsuario);
        }

        /// <summary>
        /// Obtiene los usuarios con solicitudes pendientes
        /// Seguridad: Administradores
        /// </summary>
        /// <returns>BindingList de Usuario</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public BindingList<Usuario> ObtenerUsuariosSolicitudPendiente()
        {
            UsuarioAD usuariosAD = new UsuarioAD();

            return usuariosAD.ObtenerUsuariosSolicitudPendiente();
        }

        /// <summary>
        /// Obtiene los usuarios con solicitudes pendientes en un area
        /// Seguridad: Administradores
        /// </summary>
        /// <param name="idArea">Id del area</param>
        /// <returns>BindingList de Usuario</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public BindingList<Usuario> ObtenerUsuariosSolicitudPendiente(int idArea)
        {
            UsuarioAD usuariosAD = new UsuarioAD();

            return usuariosAD.ObtenerUsuariosSolicitudPendiente(idArea);
        }
        
        /// <summary>
        /// Obtiene los usuarios del sistema
        /// Seguridad: Administradores
        /// </summary>
        /// <returns>BindingList de Usuario</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public BindingList<Usuario> ObtenerUsuarios()
        {
            UsuarioAD usuariosAD = new UsuarioAD();

            return usuariosAD.ObtenerUsuarios();
        }

        /// <summary>
        /// Obtiene los usuarios que comparten el mismo area
        /// Seguridad: Administrador
        /// </summary>
        /// <returns>BindingList de Usuario</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public BindingList<Usuario> ObtenerUsuariosPorAreaComun(String nombre)
        {
            UsuarioAD usuariosAD = new UsuarioAD();
            Usuario usuario = usuariosAD.ObtenerUsuarioPorNombre(nombre);
            return usuariosAD.ObtenerUsuariosPorAreaComun(usuario.IdUsuario);
        }


        /// <summary>
        /// Devuelve si un usuario es experto en algún area
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <param name="nombre">nombre del usuario</param>
        /// <returns>Verdadero o Falso</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public Boolean EsExperto(String nombre)
        {
            UsuarioAD usuariosAD = new UsuarioAD();
            Usuario usuario = usuariosAD.ObtenerUsuarioPorNombre(nombre);
            AreaAD areaAD = new AreaAD();
            BindingList<Area> areas = areaAD.ObtenerAreasPorExperto(usuario.IdUsuario);
            if (areas.Count == 0)
                return false;
            else
                return true;
           
        }

        #endregion


        #region mantenimientos

        /// <summary>
        /// Añade un usuario al sistema
        /// Seguridad: Publico
        /// </summary>
        /// <param name="usuario">Usuario que hace la solcitud de Experto</param>
        public int AgregarUsuario(Usuario usuario)
        {
            UsuarioAD usuarioAD = new UsuarioAD();

            if (usuarioAD.ObtenerUsuarioPorNombre(usuario.Nombre) == null)
            {
                usuario.Clave = HashHelper.CalcularHash(usuario.Clave);
                usuario.UltimaEntrada = DateTime.Today;
                return usuarioAD.AgregarUsuario(usuario);
            }
            else
            {
                throw new Exception("Existe un usuario con ese Nombre");
            }
            
           
        }

        /// <summary>
        /// Relaciona un usuario con un area
        /// Seguridad: Publico
        /// </summary>
        /// <param name="idUsuario">Id del usuario</param>
        /// <param name="idArea">Id del area</param>
        /// <param name="usuario">Usuario que hace la solcitud de Experto</param>
        public void AgregarUsuarioToArea(int idUsuario, int idArea)
        {
            UsuarioAD usuarioAD = new UsuarioAD();

            usuarioAD.AgregarUsuarioToArea(idUsuario, idArea);


        }

        /// <summary>
        /// Elimina un usuario
        /// Seguridad: Publico
        /// </summary>
        /// <param name="idUsuario">Id del usuario</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public void EliminarUsuario(int idUsuario)
        {
            UsuarioAD usuarioAD = new UsuarioAD();

            usuarioAD.EliminarUsuario(idUsuario);

        }

        /// <summary>
        /// Actualiza un usuario en las areas
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="usuario">Usuario que hace la solcitud de Experto</param>
        /// <param name="idsArea">Lista de areas</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public void ActualizarUsuario(Usuario usuario, List<int> idsArea)
        {
            UsuarioAD usuarioAD = new UsuarioAD();
            AreaAD areaAD = new AreaAD();
            Area ar=null;
            BindingList<Area> areasExperto = areaAD.ObtenerAreasPorExperto(usuario.IdUsuario);
            using (System.Transactions.TransactionScope trans = new System.Transactions.TransactionScope())
            {
                //tratamiento de areas
                foreach (int id in idsArea)
                {
                    //Si ya es experto en este area no hago nada
                    ar = areasExperto.Where(area => area.IdArea == id).SingleOrDefault();
                    if (ar == null)
                    {
                        BindingList<Area> areasSolicitud = areaAD.ObtenerAreasUsuarioPendiente(usuario.IdUsuario);
                        //Si no esta la solicitud la anoto
                        ar = areasSolicitud.Where(area => area.IdArea == id).SingleOrDefault();
                        if (ar == null)
                        {
                            //Inserto la relacion
                            usuarioAD.AgregarUsuarioToArea(usuario.IdUsuario, id);
                        }
                        //Actualizo la solicitud
                        usuarioAD.AceptarUsuarioEnArea(usuario.IdUsuario, id);
                    }
                }
                //tratamiento de usuario
                usuarioAD.ModificarUsuario(usuario);

                trans.Complete();
            }


        }

        /// <summary>
        /// Acepta la solicitud de experto de un usuario en las areas
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="idUsuario">Id de Usuario que hace la solcitud de Experto</param>
        /// <param name="idsArea">Lista de Areas</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public void AceptarUsuarioEnArea(int idUsuario, List<int> idsArea)
        {
            UsuarioAD usuarioAD = new UsuarioAD();

            using (System.Transactions.TransactionScope trans = new System.Transactions.TransactionScope())
            {
                foreach (int id in idsArea)
                    usuarioAD.AceptarUsuarioEnArea(idUsuario, id);
                trans.Complete();
            }


        }

        /// <summary>
        /// Elimina un usuario en las areas
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="idUsuario">Id de Usuario que hace la solcitud de Experto</param>
        /// <param name="idsArea">Lista de Areas</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public void EliminarUsuarioEnArea(int idUsuario, List<int> idsArea)
        {
            UsuarioAD usuarioAD = new UsuarioAD();

            using (System.Transactions.TransactionScope trans = new System.Transactions.TransactionScope())
            {
                foreach (int id in idsArea)
                    usuarioAD.EliminarUsuarioEnArea(idUsuario, id);
                trans.Complete();
            }


        }


        #endregion



        #region Autenticacion y Acceso
        /// <summary>
        /// Obtiene un usuario por NIF
        /// Seguridad: Administradores, Experto
        /// </summary>
        /// <param name="nombre">nombre del usuario</param>
        /// <returns>Usuario</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public Usuario ObtenerUsuarioPorNombre(string nombre)
        {
            UsuarioAD usuariosAD = new UsuarioAD();

            return usuariosAD.ObtenerUsuarioPorNombre(nombre);
        }



        /// <summary>
        /// Valida que un usuario tiene acceso al sistema
        /// Seguridad: Público
        /// </summary>
        /// <param name="nombre">nombre del usuario</param>
        /// <param name="clave">Clave a validar</param>
        /// <returns>Usuario</returns>
        public Usuario ValidarUsuario(string nombre, string clave)
        {
            string claveHashed;
            UsuarioAD usuarioAD = new UsuarioAD();

            claveHashed = usuarioAD.ObtenerClave(nombre);
            if (claveHashed == null)
                return null;

            if (HashHelper.ValidarClave(clave, claveHashed))
            {
                // Usuario correcto, devolvemos el registro del usuario
                return usuarioAD.ObtenerUsuarioPorNombre(nombre);
            }
            else
            {
                return null;
            }


        }

        /// <summary>
        /// Obtiene el rol del usuario dado
        /// Seguridad: Público
        /// </summary>
        /// <param name="nombre">nombre del usuario</param>
        /// <returns>Rol</returns>
        public Rol ObtenerRolUsuario(string nombre)
        {
            Usuario usuario;
            UsuarioAD usuarioAD = new UsuarioAD();

            usuario = usuarioAD.ObtenerUsuarioPorNombre(nombre);
            if (usuario != null)
                 return usuario.Rol;
                //if (usuario.Rol==0){
                //   return Rol.Experto ;
                //}else{
                //   return Rol.Administrador;
                //}
               
            else
                throw new Exception("No se pudo recuperar el Rol");
        }

        #endregion
    }
}
